	include p16f628.inc

	cblock	0x20
Delay1
bz1
Temp

TonHoehe
TonDauer
MelEnd

MelodieNummer
AdrL
AdrH

STimer0

TempStatus
TempFsr
TempPclath
TempW

Flags
	endc

#define		_TonOff		Flags,0
#define		_MelTableEnd	Flags,1
#define		_Audio		PORTB,7		; Melodieausgang
#define		_KlingelTaster	PORTB,0		; Klingeltaster gegen Masse

Constant	VMelodie	=	0x08		; Wiedergabegeschwindigkeit
Constant	MelodieAnzahl	=	0x38

; Wenn Zufall definiert, dann erfolgt die Auswahl der Melodie zufllig,
; ansonsten schn der Reihe nach

#define	Zufall			

	org 00
		goto	Start

	org 04

;	************************************************************
;	************************************************************
;	****							****
;	****	Interuptserviceroutine fr TMR0-Interupt	****
;	****	wird durch berlauf des TMR0 alle 10ms aufg.	****
;	****	Vorteiler= 1:64, TMR0 wird mit 0x64 geladen	****
;	****	Interupt alle 64*156=9984 Taktzyklen bei 4MHz	****
;	****	2 Zyklen Sprung in ISR, 15 Zyklen bis zum	****
;	****	Laden von TMR0 --> genau 10001s		****
;	****							****
;	************************************************************
;	************************************************************

	   	movwf	TempW			; wichtige Register sichern
                movf	STATUS,0
		movwf	TempStatus
                movf	FSR,0
		movwf	TempFsr
		movf	PCLATH,0
		movwf	TempPclath
		clrf	PCLATH	

		btfss	INTCON,T0IE		; RTCC-Interupt freigegeben ?
		goto	End_Int			; berprfung knnte fr dieses
						; Programm entfallen, da Interupt
						; immer freigegeben

		btfss	INTCON,T0IF		; RTCC-Interupt ?
		goto	End_Int

		bcf	INTCON,T0IF		; Interupt-flag zurcksetzen
	        movlw	0x64			; (--> Interupt alle 10ms)
		movwf	TMR0			; (genau alle 10001s)

		decfsz	STimer0,1	
		goto	End_Int			
		
		movlw	VMelodie
		movwf	STimer0			; STimer0 mit Wiedergabegeschwindigkeit
						; initialisieren
		
		decf	TonDauer,1		; Ton zu Ende?
		btfsc	STATUS,Z
		bsf	_TonOff			; Ton Ende nach 10ms*VMeldodie*Tondauer
		
End_Int         movf	TempStatus,0		; gesicherte Register zurckladen
		movwf	STATUS
                movf	TempFsr,0
		movwf	FSR
		movf	TempPclath,0
		movwf	PCLATH
       		movf	TempW,0
                bsf     INTCON,GIE
                retfie

;	************************************************************
;	************************************************************
;	****							****
;	****	Hauptprogramm					****
;	****	wartet bis Klingeltaster gedrckt		****
;	****							****
;	************************************************************
;	************************************************************

Start		call	Init_PIC

Wait_Loop	clrwdt
		btfss	_KlingelTaster		; Klingeltaster gedrckt?
		goto	Play
		
		goto	Wait_Loop		

Play		bsf	INTCON,GIE
#ifdef Zufall
		movf	TMR0,0			; TMR0 als Zufallsgenerator missbrauchen
		andlw	0x3F			; 56 Lieder, --> Bit7,6 ausblenden
		movwf	MelodieNummer
		sublw	MelodieAnzahl-1		; grer als Anzahl Lieder?
		btfsc	STATUS,C
		goto	M_Nummer_OK
		
		movlw	0x100-MelodieAnzahl			
		addwf	MelodieNummer		; berlauf --> wieder von vorn
		
#else
		incf	MelodieNummer,1
		movlw	MelodieAnzahl		; letzte Melodie gespielt?
		subwf	MelodieNummer,0
		btfsc	STATUS,Z
		clrf	MelodieNummer
		
#endif

M_Nummer_OK	movf	MelodieNummer,0
		movlw	57
		call	Get_Mel_Adr
		
	; dieser Programmteil drfte nicht durchlaufen werden, wenn
	; die berprfung der Melodie-Nummer OK ist
			
		iorlw	0
		btfsc	STATUS,Z
		goto	Adr_OK

	; falls doch Fehler, dann wird erste Melodie gespielt

		movlw	HIGH(Mel_Tab)
		movwf	AdrH
		movlw	LOW(Mel_Tab)
		movwf	AdrL

Adr_OK		call	Melodie_Out
		
		goto	Wait_Loop

;	************************************************************
;	************************************************************
;	****							****
;	****	Init_PIC					****
;	****	fhrt die notwendige Grundinitialisierung durch	****
;	****							****
;	************************************************************
;	************************************************************

Init_PIC	movlw	00000101b	; Vorteiler f. TMR0 1:64, pull-ap's enabled
		bsf	STATUS,RP0
		movwf	OPTION_REG
		bcf	STATUS,RP0
		movlw	00100000b
		movwf	INTCON		; nur TMR0-Interupt	

		bsf	STATUS,RP0		
		clrf	PIE1		; cmif Int gesp.

		clrf	VRCON		; Voltage ref. von PD, disconect

		clrf	TXSTA
		clrf	RCSTA		; Serial Port Disabled
		
		bcf	STATUS,RP0

		clrf	T1CON		; TMR1 Stop, Osz. Shut off
		clrf	T2CON		; TMR2 off

		movlw	00000111b
		movwf	CMCON		; Comperators off	
		clrf	STATUS
		clrf	PORTB
		clrf	PORTA
		bsf	STATUS,RP0

		movlw	00000000b	; alles Ausgnge
		movwf	TRISA

		movlw	00000001b	; alles Ausgnge ausser PORTB,0
		movwf	TRISB
		bcf	STATUS,RP0

		movlw	0
		movwf	MelodieNummer

		retlw	0

;	************************************************************
;	************************************************************
;	****							****
;	****	Ton_Out						****
;	****	erzeugt einen Ton mit der in TonHoehe		****
;	****    spezifizierten Periodendauer			****
;	****	F= 1/(TonHoehe+150)*100000 in Hz		****
;	****	fr die Zeit 10ms*TonDauer*VMelodie		****
;	****							****
;	************************************************************
;	************************************************************


;	w*5s Pause
Ton_Pause	movwf	Delay1
TP_Next		clrwdt
		nop
		decfsz	Delay1,1
		goto	TP_Next
		return
		
Ton_Out		movlw	0x64
		movwf	TMR0		; TMR0 neu Starten
		movlw	VMelodie
		movwf	STimer0

WTon_Out	movf	TonHoehe,0
		iorlw	0
		btfss	STATUS,Z	; bei Tonhhe=0 keine Ausgabe
					; berprfung knnte auch in
					; Melodie_Out erfolgen
		
		bsf	_Audio		; Ausgang setzen
		call	Ton_Pause
		movlw	0x94
		call	Ton_Pause
		bcf	_Audio		; Ausgang wieder zurcksetzen
		movf	TonHoehe,0
		call	Ton_Pause
		nop			; fr genaue Laufzeit 
		nop			; Thigh=Tlow
		movlw	0x92
		call	Ton_Pause
		
		btfsc	_TonOff		; wird in ISR gesetzt
		return
		
		goto	WTon_Out

;	************************************************************
;	************************************************************
;	****							****
;	****	Melodie_Out 					****
;	****	spielt die Melodie, welche an AdrH,AdrL		****
;	****    im programmspeicher abgelegt ist		****
;	****							****
;	************************************************************
;	************************************************************

Melodie_Out	call	Get_Ton		; aktueller Ton in w
		movwf	TonDauer
		movwf	MelEnd
		andlw	0x0f
		movwf	TonHoehe	; Tonhhe jetzt OK
		iorlw	0
		btfsc	STATUS,Z	; =0 ? --> Pause
		goto	T_Dauer	
		call	Get_Periode
		movwf	TonHoehe
		
T_Dauer		rrf	TonDauer,1
		rrf	TonDauer,1
		rrf	TonDauer,1
		rrf	TonDauer,1	
		movlw	0x07
		andwf	TonDauer,1	; Nur Bits 2,1,0 interessant
		incf	TonDauer,1
		
		bcf	_TonOff
		call	Ton_Out		; ausgeben
		
		btfsc	MelEnd,7
		retlw	0		; Ende

		incfsz	AdrL,1
		goto	Melodie_Out
		incf	AdrH,1
		goto	Melodie_Out
		
;	************************************************************
;	************************************************************
;	****							****
;	****	Get_Mel_adr					****
;	****	Liefert die Anfangsadresse der in w 		****
;	****	spezifizierten Melodie in AdrH und Adrl		****
;	****	w=0, wenn MelodieNr. gefunden			****
;	****	w=1, wenn Vorzeitig Tabellenende erreicht wird	****
;	****							****
;	************************************************************
;	************************************************************

;	eine andere Mglichkeit whre die Anfangsadressen ebenfalls
;	in einer Tabelle abzulegen

Get_Mel_Adr	movwf	bz1		; Liednummer zwischenspeichern
		incf	bz1,1
		bcf	_MelTableEnd
		movlw	HIGH(Mel_Tab)	; High-Byte Anfang Melodietabelle
		movwf	AdrH 
		movlw	Low(Mel_Tab)	; Low-Byte Anfang Melodietabelle
		movwf	AdrL
		
Next_Mel_Nr	decf	bz1,1
		btfss	STATUS,Z
		goto	Next_Mel_Byte

		call	Get_Ton
		sublw	0x80
		btfss	STATUS,C	
		retlw	1		; Ende der Tabelle
		
		retlw	0		
		
Next_Mel_Byte	call	Get_Ton
		movwf	Temp		; zwischenspeichern
		clrwdt
		
		incfsz	AdrL,1		; Low Teil Adresse erhhen
		goto	No_Overrun	; Overrun?
		incf	AdrH,1		; High Teil Adresse erhhen		

No_Overrun	movf	Temp,0

		sublw	0x80
		btfsc	STATUS,C	; gesetzt, wenn w > 0x80
					; = Lied Ende
		goto	Norm_Ton	; normaler Ton
		
		btfsc	_MelTableEnd
		retlw	1		; Tablellenende vorzeitig erreicht
		
		bsf	_MelTableEnd
		goto	Next_Mel_Nr
		
Norm_Ton	bcf	_MelTableEnd
		goto	Next_Mel_Byte


;	************************************************************
;	************************************************************
;	****							****
;	****	Get_Periode liefert 1/10 - 150 der Periodendauer****
;	****	der jeweiligen Note				****
;	****    die Notennummer	(1..0xF) wird in w bergeben	****
;	****							****
;	************************************************************
;	************************************************************

;	die Frequenzangaben sind von einem Beispiel von 
;	M. Baudisch, horace@earthling.net bernommen
;	die als Kommentar angegebenen Frequenzen sind das "Sollma"
;	durch Rundungsfehler weichen die realen Frequenzen leicht ab
;	F=1/(W*1+150)*100000 in Hz

Get_Periode	clrf	PCLATH
		andlw	0x0f		; Sicherstellen, das w max. 15
		addwf	PCL,1		; Achtung funktioniert nur, wenn
					; Tablelle im Adressbereich
					; 0x0000 .. 0x00FF liegt
; 1 Periode
; *10+1500 --> in s

		retlw	0x07		; 638Hz (kann nicht zum Einsatz kommen,
					;        da Tonhhe=0 --> Pause)
		retlw	0x10		; 602Hz
		retlw	0x1A		; 568Hz
		retlw	0x25		; 536Hz
		retlw	0x30		; 505Hz
		retlw	0x3C		; 477Hz
		retlw	0x48		; 451Hz
		retlw	0x55		; 425Hz
		retlw	0x63		; 402Hz
		retlw	0x72		; 379Hz
		retlw	0x81		; 358Hz
		retlw	0x92		; 338Hz
		retlw	0xA3		; 319Hz
		retlw	0xB6		; 301Hz
		retlw	0xCA		; 284Hz
		retlw	0xDE		; 269Hz

;	************************************************************
;	************************************************************
;	****							****
;	****	Melodientabelle					****
;	****	Get_Ton liefert einen Ton einer Melodie 	****
;	****							****
;	****    Die Position des Tones wird in AdrL und AdrH	****
;	****	spezifiziert.
;	****							****
;	************************************************************
;	************************************************************

;	die Melodien sind ebenfalls von einem Beispiel von 
;	M. Baudisch, horace@earthling.net bernommen

;  Das Tonbyte hat folgenden Aufbau
;  Bit
;  7  6  5  4  3  2  1  0
;  |  |__|__|  |__|__|__|_ Tonhhe ( 1..15 (G..B') incl. aller Halbtonschritte
;  |     |                 wenn 0, kein Ton sondern Pause
;  |     |
;  |     |________________ Tonlnge in Achtelnoten
;  |
;  |______________________ Stop-Bit

;  Grundidee und Tonformat von Steffen Lehmann 'Eine Melodieklingel mit vielen Melodien'
;  (eine Schaltung mit einem Z80)
;  Zeitschrift 'Funkamateur' 4/86 S. 203

; !!! Achtung, wird ein anderer Prozessor mit mehr als 2kB Programmspeicher
; verwendet sollte nach Aufruf der Routine PCLATH wieder auf den Wert vor Aufruf
; gesetzt werden

Get_Ton		movf	AdrH,0
		movwf	PCLATH
		movf	AdrL,0
		movwf	PCL

Mel_Tab		retlw	0x21
		retlw	0x00
		retlw	0x26
		retlw	0x00
		retlw	0x16
		retlw	0x18
		retlw	0x2A
		retlw	0x00
		retlw	0x26
		retlw	0x00
		retlw	0x6D
		retlw	0x00
		retlw	0x4A
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x2B
		retlw	0x00
		retlw	0x1D
		retlw	0x1B
		retlw	0x1A
		retlw	0x1B
		retlw	0x2D
		retlw	0x00
		retlw	0x18
		retlw	0x16
		retlw	0x18
		retlw	0x1A
		retlw	0x18
		retlw	0x10
		retlw	0x21
		retlw	0x00
		retlw	0x26
		retlw	0x00
		retlw	0x16
		retlw	0x18
		retlw	0x2A
		retlw	0x00
		retlw	0x26
		retlw	0x00
		retlw	0x6D
		retlw	0x00
		retlw	0x4A
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x1B
		retlw	0x1D
		retlw	0x1A
		retlw	0x1B
		retlw	0x38
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0xD6
		retlw	0x03
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x07
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x3A
		retlw	0x08
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x3E
		retlw	0x0A
		retlw	0x00
		retlw	0x07
		retlw	0x00
		retlw	0xAF
		retlw	0x21
		retlw	0x00
		retlw	0x36
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x26
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x38
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x28
		retlw	0x00
		retlw	0x1A
		retlw	0x18
		retlw	0x36
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x2D
		retlw	0x00
		retlw	0x7F
		retlw	0x10
		retlw	0x2F
		retlw	0x00
		retlw	0x3D
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x26
		retlw	0x00
		retlw	0x38
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x28
		retlw	0x00
		retlw	0x1A
		retlw	0x18
		retlw	0x36
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x23
		retlw	0x00
		retlw	0x21
		retlw	0x00
		retlw	0xE6
		retlw	0x07
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x07
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x27
		retlw	0x00
		retlw	0x0C
		retlw	0x20
		retlw	0x27
		retlw	0x00
		retlw	0x8C
		retlw	0x06
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x16
		retlw	0x10
		retlw	0x06
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x16
		retlw	0x10
		retlw	0x16
		retlw	0x10
		retlw	0x38
		retlw	0x16
		retlw	0x10
		retlw	0x16
		retlw	0x10
		retlw	0x0A
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x1A
		retlw	0x10
		retlw	0x0A
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x1A
		retlw	0x10
		retlw	0x1A
		retlw	0x10
		retlw	0x3B
		retlw	0x1A
		retlw	0x10
		retlw	0x1A
		retlw	0x10
		retlw	0x1A
		retlw	0x10
		retlw	0x5D
		retlw	0x00
		retlw	0x0B
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x18
		retlw	0x10
		retlw	0x11
		retlw	0x10
		retlw	0x1D
		retlw	0x10
		retlw	0x0F
		retlw	0x00
		retlw	0x0D
		retlw	0x00
		retlw	0x0B
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x18
		retlw	0x10
		retlw	0x18
		retlw	0x10
		retlw	0x98
		retlw	0x6A
		retlw	0x00
		retlw	0x6A
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x3C
		retlw	0x30
		retlw	0x0A
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x09
		retlw	0x00
		retlw	0xB8
		retlw	0x2C
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x2D
		retlw	0x00
		retlw	0x2F
		retlw	0x00
		retlw	0x2F
		retlw	0x00
		retlw	0x2D
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x28
		retlw	0x00
		retlw	0x28
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x4C
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x4A
		retlw	0x10
		retlw	0x2C
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x2D
		retlw	0x00
		retlw	0x2F
		retlw	0x00
		retlw	0x2F
		retlw	0x00
		retlw	0x2D
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x28
		retlw	0x00
		retlw	0x28
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x4A
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0xD8
		retlw	0x03
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x37
		retlw	0x3A
		retlw	0x0E
		retlw	0x20
		retlw	0x1C
		retlw	0x10
		retlw	0x2C
		retlw	0x00
		retlw	0xA7
		retlw	0x36
		retlw	0x00
		retlw	0x7B
		retlw	0x00
		retlw	0x0D
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x0D
		retlw	0x00
		retlw	0x3F
		retlw	0x00
		retlw	0x3F
		retlw	0x00
		retlw	0x3F
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x0B
		retlw	0x00
		retlw	0x5F
		retlw	0x00
		retlw	0x0D
		retlw	0x00
		retlw	0x5B
		retlw	0x10
		retlw	0x1D
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x1F
		retlw	0x00
		retlw	0x0D
		retlw	0x00
		retlw	0x3B
		retlw	0x00
		retlw	0x1D
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x2F
		retlw	0x10
		retlw	0x1D
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x2F
		retlw	0x10
		retlw	0x0D
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x0D
		retlw	0x00
		retlw	0xDB
		retlw	0x1F
		retlw	0x00
		retlw	0x1C
		retlw	0x30
		retlw	0x1F
		retlw	0x00
		retlw	0x1C
		retlw	0x30
		retlw	0x1A
		retlw	0x00
		retlw	0x18
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0xB8
		retlw	0x11
		retlw	0x00
		retlw	0x13
		retlw	0x00
		retlw	0x35
		retlw	0x00
		retlw	0x15
		retlw	0x00
		retlw	0x18
		retlw	0x00
		retlw	0x36
		retlw	0x00
		retlw	0x16
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x38
		retlw	0x00
		retlw	0x18
		retlw	0x00
		retlw	0x16
		retlw	0x00
		retlw	0x35
		retlw	0x35
		retlw	0x00
		retlw	0x38
		retlw	0x00
		retlw	0x18
		retlw	0x00
		retlw	0x16
		retlw	0x00
		retlw	0x35
		retlw	0x00
		retlw	0x15
		retlw	0x00
		retlw	0x18
		retlw	0x00
		retlw	0x36
		retlw	0x00
		retlw	0x36
		retlw	0x00
		retlw	0x33
		retlw	0x00
		retlw	0x38
		retlw	0x00
		retlw	0x35
		retlw	0xD5
		retlw	0x47
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x07
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x07
		retlw	0x00
		retlw	0x0E
		retlw	0x00
		retlw	0x0C
		retlw	0x20
		retlw	0x0C
		retlw	0x20
		retlw	0x9C
		retlw	0x13
		retlw	0x17
		retlw	0x18
		retlw	0x3A
		retlw	0x30
		retlw	0x13
		retlw	0x17
		retlw	0x18
		retlw	0x3A
		retlw	0x30
		retlw	0x13
		retlw	0x17
		retlw	0x18
		retlw	0x3A
		retlw	0x37
		retlw	0x33
		retlw	0x37
		retlw	0xB5
		retlw	0x53
		retlw	0x17
		retlw	0x3A
		retlw	0x3F
		retlw	0x3C
		retlw	0x1F
		retlw	0x1C
		retlw	0xBA
		retlw	0x3D
		retlw	0x00
		retlw	0x3D
		retlw	0x00
		retlw	0x3D
		retlw	0x00
		retlw	0x2D
		retlw	0x0F
		retlw	0x00
		retlw	0x28
		retlw	0x08
		retlw	0x00
		retlw	0x28
		retlw	0x08
		retlw	0x00
		retlw	0x58
		retlw	0x40
		retlw	0x28
		retlw	0x08
		retlw	0x00
		retlw	0x28
		retlw	0x08
		retlw	0x00
		retlw	0x58
		retlw	0x40
		retlw	0x26
		retlw	0x06
		retlw	0x00
		retlw	0x26
		retlw	0x06
		retlw	0x00
		retlw	0xD6
		retlw	0x13
		retlw	0x20
		retlw	0x15
		retlw	0x20
		retlw	0x17
		retlw	0x00
		retlw	0x1A
		retlw	0x1C
		retlw	0x00
		retlw	0x1A
		retlw	0x1F
		retlw	0x20
		retlw	0x1C
		retlw	0x20
		retlw	0x1A
		retlw	0xB7
		retlw	0x1C
		retlw	0x10
		retlw	0x1C
		retlw	0x10
		retlw	0x0F
		retlw	0x00
		retlw	0x1F
		retlw	0x10
		retlw	0x0F
		retlw	0x00
		retlw	0x0D
		retlw	0x00
		retlw	0x0D
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x07
		retlw	0x00
		retlw	0xA8
		retlw	0x17
		retlw	0x18
		retlw	0x3A
		retlw	0x27
		retlw	0x00
		retlw	0x27
		retlw	0x00
		retlw	0x37
		retlw	0x95
		retlw	0x08
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x28
		retlw	0x00
		retlw	0x25
		retlw	0x30
		retlw	0x08
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x2F
		retlw	0x00
		retlw	0x0E
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x2E
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0xDA
		retlw	0x33
		retlw	0x08
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x3C
		retlw	0x3F
		retlw	0x3C
		retlw	0x3A
		retlw	0xB8
		retlw	0x0F
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x1E
		retlw	0x10
		retlw	0x1C
		retlw	0x10
		retlw	0x0A
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x1A
		retlw	0x10
		retlw	0x0A
		retlw	0x00
		retlw	0x37
		retlw	0x30
		retlw	0x0A
		retlw	0x00
		retlw	0x18
		retlw	0x10
		retlw	0x08
		retlw	0x00
		retlw	0x35
		retlw	0x30
		retlw	0x0C
		retlw	0x00
		retlw	0x1A
		retlw	0x10
		retlw	0x0A
		retlw	0x00
		retlw	0xB7
		retlw	0x45
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x02
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x5E
		retlw	0x10
		retlw	0xBA
		retlw	0x15
		retlw	0x1E
		retlw	0x1C
		retlw	0x1A
		retlw	0x1C
		retlw	0x1A
		retlw	0x17
		retlw	0x15
		retlw	0x72
		retlw	0x15
		retlw	0x1E
		retlw	0x1C
		retlw	0x1A
		retlw	0x1A
		retlw	0x19
		retlw	0x1A
		retlw	0xFC
		retlw	0x03
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x25
		retlw	0x00
		retlw	0x23
		retlw	0x00
		retlw	0x28
		retlw	0x00
		retlw	0x47
		retlw	0x20
		retlw	0x03
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x25
		retlw	0x00
		retlw	0x23
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0xC8
		retlw	0x28
		retlw	0x00
		retlw	0x1A
		retlw	0x08
		retlw	0x00
		retlw	0x25
		retlw	0x00
		retlw	0x25
		retlw	0x00
		retlw	0x15
		retlw	0x03
		retlw	0x00
		retlw	0x15
		retlw	0x06
		retlw	0x00
		retlw	0xC5
		retlw	0x15
		retlw	0x12
		retlw	0x50
		retlw	0x12
		retlw	0x23
		retlw	0x15
		retlw	0x0E
		retlw	0x20
		retlw	0x0E
		retlw	0x20
		retlw	0xBA
		retlw	0x1A
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x08
		retlw	0x36
		retlw	0x20
		retlw	0x1A
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x08
		retlw	0x36
		retlw	0x20
		retlw	0x1B
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x1B
		retlw	0x00
		retlw	0x1F
		retlw	0x00
		retlw	0x1D
		retlw	0x00
		retlw	0x1B
		retlw	0x00
		retlw	0x1B
		retlw	0x9A
		retlw	0x28
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x23
		retlw	0x10
		retlw	0x38
		retlw	0x00
		retlw	0x25
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x25
		retlw	0x40
		retlw	0x27
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x2A
		retlw	0x10
		retlw	0x27
		retlw	0x10
		retlw	0x28
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0xB8
		retlw	0x35
		retlw	0x00
		retlw	0x25
		retlw	0x05
		retlw	0x38
		retlw	0x00
		retlw	0x28
		retlw	0x08
		retlw	0x00
		retlw	0x3A
		retlw	0x00
		retlw	0x2D
		retlw	0x0D
		retlw	0xDC
		retlw	0x0C
		retlw	0x00
		retlw	0x0B
		retlw	0x00
		retlw	0x0C
		retlw	0x20
		retlw	0x0C
		retlw	0x00
		retlw	0x0B
		retlw	0x00
		retlw	0x0C
		retlw	0x20
		retlw	0x0C
		retlw	0x00
		retlw	0x0B
		retlw	0x00
		retlw	0x0C
		retlw	0x20
		retlw	0x0A
		retlw	0x20
		retlw	0x98
		retlw	0x3C
		retlw	0x00
		retlw	0x3C
		retlw	0x00
		retlw	0x3C
		retlw	0x00
		retlw	0x2F
		retlw	0x0D
		retlw	0x00
		retlw	0x5A
		retlw	0x30
		retlw	0x3C
		retlw	0x00
		retlw	0x3A
		retlw	0x00
		retlw	0x38
		retlw	0x00
		retlw	0x28
		retlw	0x07
		retlw	0x00
		retlw	0xB7
		retlw	0x03
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x28
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x2D
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x07
		retlw	0x00
		retlw	0x3A
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0xA8
		retlw	0x13
		retlw	0x38
		retlw	0x00
		retlw	0x08
		retlw	0x0A
		retlw	0x0C
		retlw	0x0D
		retlw	0x00
		retlw	0x3F
		retlw	0x00
		retlw	0x1C
		retlw	0x00
		retlw	0x1C
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x1F
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x1F
		retlw	0x00
		retlw	0x0C
		retlw	0x0D
		retlw	0x0C
		retlw	0x0A
		retlw	0x00
		retlw	0x98
		retlw	0x23
		retlw	0x00
		retlw	0x4F
		retlw	0x00
		retlw	0x0E
		retlw	0x00
		retlw	0x2F
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x4D
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x2D
		retlw	0x00
		retlw	0x27
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x98
		retlw	0x0E
		retlw	0x0D
		retlw	0x00
		retlw	0x1E
		retlw	0x15
		retlw	0x00
		retlw	0x1C
		retlw	0x15
		retlw	0x00
		retlw	0x3C
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x0A
		retlw	0x09
		retlw	0x00
		retlw	0x1A
		retlw	0x15
		retlw	0x00
		retlw	0x1C
		retlw	0x15
		retlw	0x00
		retlw	0x3A
		retlw	0x00
		retlw	0x99
		retlw	0x0C
		retlw	0x20
		retlw	0x2A
		retlw	0x20
		retlw	0x07
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x2B
		retlw	0x00
		retlw	0xAC
		retlw	0x0A
		retlw	0x00
		retlw	0x0F
		retlw	0x10
		retlw	0x17
		retlw	0x10
		retlw	0x18
		retlw	0x10
		retlw	0x0A
		retlw	0x00
		retlw	0x0F
		retlw	0x10
		retlw	0x3C
		retlw	0x20
		retlw	0x0C
		retlw	0x00
		retlw	0x0E
		retlw	0x10
		retlw	0x1F
		retlw	0x10
		retlw	0x1A
		retlw	0x10
		retlw	0x08
		retlw	0x00
		retlw	0x0C
		retlw	0x10
		retlw	0xBA
		retlw	0x47
		retlw	0x10
		retlw	0x47
		retlw	0x10
		retlw	0x07
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x07
		retlw	0x00
		retlw	0x3A
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x45
		retlw	0x10
		retlw	0x45
		retlw	0x10
		retlw	0x05
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0xD7
		retlw	0x0A
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x1C
		retlw	0x10
		retlw	0x1A
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x18
		retlw	0x10
		retlw	0x07
		retlw	0x00
		retlw	0xC8
		retlw	0x13
		retlw	0x00
		retlw	0x17
		retlw	0x00
		retlw	0x5A
		retlw	0x00
		retlw	0x1C
		retlw	0x00
		retlw	0x3A
		retlw	0x00
		retlw	0x38
		retlw	0x00
		retlw	0x37
		retlw	0x00
		retlw	0x35
		retlw	0x00
		retlw	0x33
		retlw	0x00
		retlw	0x3A
		retlw	0x00
		retlw	0x3C
		retlw	0x00
		retlw	0x3A
		retlw	0x00
		retlw	0x38
		retlw	0x00
		retlw	0x37
		retlw	0x00
		retlw	0xF5
		retlw	0x13
		retlw	0x00
		retlw	0x15
		retlw	0x00
		retlw	0x17
		retlw	0x00
		retlw	0x18
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x1C
		retlw	0x00
		retlw	0x18
		retlw	0x00
		retlw	0x1F
		retlw	0x00
		retlw	0x1C
		retlw	0x00
		retlw	0xDA
		retlw	0x1C
		retlw	0x00
		retlw	0x19
		retlw	0x00
		retlw	0x1C
		retlw	0x00
		retlw	0x19
		retlw	0x00
		retlw	0x1C
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x1A
		retlw	0x20
		retlw	0x1A
		retlw	0x00
		retlw	0x17
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x17
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0xB9
		retlw	0x2D
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x0D
		retlw	0x00
		retlw	0x0B
		retlw	0x00
		retlw	0x1A
		retlw	0x10
		retlw	0x16
		retlw	0x10
		retlw	0x16
		retlw	0x10
		retlw	0x18
		retlw	0x10
		retlw	0x11
		retlw	0x10
		retlw	0x11
		retlw	0x10
		retlw	0x1A
		retlw	0x10
		retlw	0x16
		retlw	0x10
		retlw	0x96
		retlw	0x3E
		retlw	0x30
		retlw	0x3A
		retlw	0x30
		retlw	0x05
		retlw	0x00
		retlw	0x6A
		retlw	0x30
		retlw	0x3E
		retlw	0x30
		retlw	0x3A
		retlw	0x30
		retlw	0x0C
		retlw	0x00
		retlw	0x6C
		retlw	0x30
		retlw	0x3E
		retlw	0x30
		retlw	0x3A
		retlw	0x30
		retlw	0x0F
		retlw	0x00
		retlw	0x6F
		retlw	0x10
		retlw	0x0F
		retlw	0x00
		retlw	0x2E
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x09
		retlw	0x00
		retlw	0xEA
		retlw	0x38
		retlw	0x20
		retlw	0x17
		retlw	0x10
		retlw	0x1A
		retlw	0x10
		retlw	0x38
		retlw	0x50
		retlw	0x3C
		retlw	0x20
		retlw	0x1A
		retlw	0x10
		retlw	0x1D
		retlw	0x10
		retlw	0x3C
		retlw	0x40
		retlw	0x08
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x1C
		retlw	0x10
		retlw	0x1C
		retlw	0x10
		retlw	0x1C
		retlw	0x10
		retlw	0x0A
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x1D
		retlw	0x10
		retlw	0x0A
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x9A
		retlw	0x25
		retlw	0x00
		retlw	0x3A
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x0E
		retlw	0x0E
		retlw	0x00
		retlw	0x4A
		retlw	0x00
		retlw	0x25
		retlw	0x00
		retlw	0x08
		retlw	0x0E
		retlw	0x00
		retlw	0x2C
		retlw	0x00
		retlw	0x25
		retlw	0x00
		retlw	0x25
		retlw	0x00
		retlw	0x07
		retlw	0x08
		retlw	0x00
		retlw	0xCA
		retlw	0x07
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x13
		retlw	0x10
		retlw	0x13
		retlw	0x10
		retlw	0x2F
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x0E
		retlw	0x00
		retlw	0x4C
		retlw	0x10
		retlw	0x1C
		retlw	0x10
		retlw	0x0A
		retlw	0x00
		retlw	0x27
		retlw	0x00
		retlw	0x15
		retlw	0x10
		retlw	0x13
		retlw	0x10
		retlw	0xD5
		retlw	0x18
		retlw	0x10
		retlw	0x1D
		retlw	0x10
		retlw	0x08
		retlw	0x00
		retlw	0x05
		retlw	0x00
		retlw	0x18
		retlw	0x10
		retlw	0x06
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x28
		retlw	0x00
		retlw	0x15
		retlw	0x40
		retlw	0x08
		retlw	0x00
		retlw	0x18
		retlw	0x10
		retlw	0x0A
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x16
		retlw	0x10
		retlw	0x05
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0xB1
		retlw	0x06
		retlw	0x00
		retlw	0x16
		retlw	0x10
		retlw	0x06
		retlw	0x00
		retlw	0x16
		retlw	0x10
		retlw	0x06
		retlw	0x00
		retlw	0x26
		retlw	0x00
		retlw	0x13
		retlw	0x10
		retlw	0x06
		retlw	0x00
		retlw	0x1B
		retlw	0x10
		retlw	0x0B
		retlw	0x00
		retlw	0x1B
		retlw	0x10
		retlw	0x0B
		retlw	0x00
		retlw	0x1F
		retlw	0x00
		retlw	0x0D
		retlw	0x00
		retlw	0x9B
		retlw	0x18
		retlw	0x00
		retlw	0x18
		retlw	0x00
		retlw	0x38
		retlw	0x10
		retlw	0x1A
		retlw	0x00
		retlw	0x1A
		retlw	0x00
		retlw	0x3A
		retlw	0x10
		retlw	0x1C
		retlw	0x00
		retlw	0x1D
		retlw	0x00
		retlw	0x1F
		retlw	0x00
		retlw	0x18
		retlw	0x00
		retlw	0x18
		retlw	0x00
		retlw	0x17
		retlw	0x00
		retlw	0xA8
		retlw	0x13
		retlw	0x00
		retlw	0x28
		retlw	0x10
		retlw	0x1C
		retlw	0x00
		retlw	0x3F
		retlw	0x10
		retlw	0x18
		retlw	0x00
		retlw	0x5C
		retlw	0x10
		retlw	0x13
		retlw	0x00
		retlw	0x38
		retlw	0x10
		retlw	0x1A
		retlw	0x00
		retlw	0x3C
		retlw	0x10
		retlw	0x1D
		retlw	0x00
		retlw	0xBA
		retlw	0x21
		retlw	0x00
		retlw	0x46
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x0B
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x2D
		retlw	0x00
		retlw	0x2D
		retlw	0x00
		retlw	0x0D
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x0B
		retlw	0x00
		retlw	0x28
		retlw	0x00
		retlw	0xA8
		retlw	0x16
		retlw	0x10
		retlw	0x08
		retlw	0x00
		retlw	0x23
		retlw	0x00
		retlw	0x06
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x26
		retlw	0x00
		retlw	0x18
		retlw	0x10
		retlw	0x03
		retlw	0x00
		retlw	0x36
		retlw	0x30
		retlw	0x1B
		retlw	0x00
		retlw	0x0F
		retlw	0x00
		retlw	0x2D
		retlw	0x00
		retlw	0x0B
		retlw	0x00
		retlw	0xDB
		retlw	0x28
		retlw	0x00
		retlw	0x4D
		retlw	0x00
		retlw	0x08
		retlw	0x00
		retlw	0x25
		retlw	0x00
		retlw	0x26
		retlw	0x00
		retlw	0x48
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0xA8
		retlw	0x23
		retlw	0x00
		retlw	0x03
		retlw	0x00
		retlw	0x07
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x2A
		retlw	0x00
		retlw	0x0C
		retlw	0x00
		retlw	0x0A
		retlw	0x00
		retlw	0x27
		retlw	0x00
		retlw	0xA3
		retlw	0x00
		retlw	0x43
		retlw	0x10
		retlw	0x03
		retlw	0x00
		retlw	0x17
		retlw	0x10
		retlw	0x1A
		retlw	0x10
		retlw	0x5F
		retlw	0x30
		retlw	0x1F
		retlw	0x10
		retlw	0x3F
		retlw	0x10
		retlw	0x0F
		retlw	0x00
		retlw	0x1C
		retlw	0x10
		retlw	0x18
		retlw	0x10
		retlw	0xCA
		retlw	0xff		; zweimal hintereinander Bit7 gesetzt
					; --> Tabellenende
	end
